
// ************************ BEGIN_COPYRIGHT_NOTICE ************************
//
// This source code copyright (c) 1995 by Eric Dill and Paul Keister
//
// Permission to use, copy, modify, distribute, and sell this software
// and its documentation for any purpose is hereby granted without
// fee, provided that this copyright notice appear in all copies
// and that the author's name not be used in advertising or publicity
// pertaining to distribution of the software without specific, written
// prior permission.  The author makes no representations about the
// suitability of this software for any purpose.  It is provided "as
// is" without express or implied warranty.
//
// Please send all bug fixes, ideas, and comments to:
//
//             Via Email to CompuServe 75061,3310 and 71242,1145
//             Or post to the Borland OWL2:Print/Doc/View Forum on CompuServe
//
// ************************* END_COPYRIGHT_NOTICE *************************


#include "demo.h"
#pragma hdrstop


// ------------- Custom Printer Object --------------
//
//    provides access to TPrinter::ExecPrintDialog()
//
// --------------------------------------------------

class Printer : public TPrinter
{
   public:
      Printer() : TPrinter() {}

      bool ExecPrintDialog( TWindow *parent )
         {
            // no print to file, page numbers, or page selection
            Data->Flags |= PD_HIDEPRINTTOFILE | PD_NOPAGENUMS
                                              | PD_NOSELECTION;
            // no collate copies
            Data->Flags &= ~PD_COLLATE;

            return TPrinter::ExecPrintDialog( parent );
         }
};


//
// ------------ Main Application Object -------------
//

class TApp : public TApplication
{
   private:

   public:
      TApp();
      virtual ~TApp();

      virtual void InitMainWindow();
      virtual void InitInstance();
};


//
// --------------- Frame Window Object ---------------
//

class SDIFrame : public TDecoratedFrame
{
   public:
      SDIFrame (TWindow *parent, const char far *title, TWindow *clientWnd, BOOL trackMenuSelection = FALSE, TModule *module = 0)
            :  TDecoratedFrame(parent, title, clientWnd, trackMenuSelection, module),
               size( 600, 300 )

         {
            // Override the default window style for the main window.
            Attr.Style  = WS_BORDER | WS_SYSMENU     | WS_CAPTION
                                    | WS_MAXIMIZEBOX | WS_MINIMIZEBOX
                                    | WS_VISIBLE     | WS_SIZEBOX;
            AssignMenu(DEMO);
            Attr.AccelTable = DEMO;
            Attr.X = 50;
            Attr.Y = 50;
            Attr.W = Attr.X + size.cx;
            Attr.H = Attr.Y + size.cy;
         }

      ~SDIFrame ()
         {
         }

      TSize size;

   protected:
      virtual void SetupWindow();

   protected:
      void Cm1cmfileprint();
      void Cm1cmexit();
      void EvSize( uint sizeType, TSize& Size )
         {
            size = Size;
            TDecoratedFrame::EvSize( sizeType, Size );
            ClientWnd->Invalidate();
         }

      DECLARE_RESPONSE_TABLE(SDIFrame);
};


//
// --------------- Main Window Object ----------------
//

class PrtStreamWindow : public TWindow
{
   public:
      PrtStreamWindow( TWindow *parent )
         : TWindow( parent, "Print Stream Demo" )  {}

      void Paint( TDC& dc, bool erase, TRect& rect )
         {
            TWindow::Paint( dc, true, rect );

            SDIFrame *frame = dynamic_cast< SDIFrame *>(Parent);

            TResId   ResId( "prtStream" );
            TDib     bm( GetModule()->GetInstance(), ResId);

            TSize    dSize = frame->size;
                     if ( dSize.cx < 6 * dSize.cy )
                        dSize.cy = dSize.cx / 6;
                     else
                        dSize.cx = dSize.cy * 6;

            TSize    bmSize( bm.Width(),    bm.Height() );
            TPoint   dOrg(  (frame->size.cx-dSize.cx) / 2,
                            (frame->size.cy-dSize.cy) / 2 );

            TRect    src(    TPoint( 0, 0), bmSize );
            TRect    dest(   dOrg,          dSize  );

            dc.StretchDIBits( dest, src, bm );
         }
};


void SDIFrame::SetupWindow()
{
   TDecoratedFrame::SetupWindow();
}

DEFINE_RESPONSE_TABLE1(SDIFrame, TDecoratedFrame)
   EV_COMMAND(CM_FILEPRINT, Cm1cmfileprint),
   EV_COMMAND(CM_EXIT, Cm1cmexit),
   EV_WM_SIZE,
END_RESPONSE_TABLE;

TApp::TApp()
{
}

void TApp::InitInstance()
{
   EnableCtl3d();
   TApplication::InitInstance();
}


TApp::~TApp()
{
}


//
// Application intialization
//

void TApp::InitMainWindow()
{
   TWindow *pClientWnd = new PrtStreamWindow( 0 );

   SDIFrame *frame = new SDIFrame(0, "Print Stream Demo", pClientWnd, FALSE);
   nCmdShow = SW_SHOWNORMAL;

   frame->SetIcon( this, "demo" );

   SetMainWindow( frame );
}


int OwlMain (int , char* [])
{
    TApp App;
    int result;

    result = App.Run();

    return result;
}

char gbuf[10240];

void SDIFrame::Cm1cmfileprint()
{
   const char *h_font = "Helvetica";
   const char *f_font = "Times New Roman";
   const char *n_font = "Courier New";

   const  header = VARIABLE_PITCH | FF_ROMAN;
   const  footer = VARIABLE_PITCH | FF_SWISS;
   const  normal = FIXED_PITCH    | FF_MODERN;

   // Note: Positive font heights are interpeted as point
   //       size (72pts/inch) within the prtStream class

   TFont  headerFont( h_font, 14, 0, 0, 0, FW_BOLD,    header );
   TFont  footerFont( f_font, 10, 0, 0, 0, FW_BOLD,    footer );
   TFont  normalFont( n_font,  8, 0, 0, 0, FW_REGULAR, normal );

   TResId bmp( "prtStream" );    // create a bitmap resource ID

   // Here are but a few ways to init the stream
   // 1) - as before, defaults to a strstream
// prtStream prt;

   // 2) - like before but an explicitly passed strstreambuf
   //      to a buffer global to us (usefull for debugging).
// prtStream prt( &strstreambuf( gbuf, sizeof(gbuf), gbuf ) );

   // 3) - not so much like before, this will "spool" to a file
   //      before printing, the spoolfilebuf is a specialized filebuf
   //      that creates a temparary spool file and deletes it after use!
   spoolfilebuf fbuf ;              // create a spool file
   prtStream    prt( &fbuf );       // create a disk based print stream

   prt << setLeftMargin(   1.00 )
       << setTopMargin(    1.00 )
       << setBottomMargin( 1.00 );

	prt << startHeader()
       << setFont( headerFont )     // set a font for the header
       << setPrintPos(  0.5, 3.188, true )
       << "prtStream demo"
       << bitMap( bmp, 7.0, 0.0 )   // draw bitmap at printers resolution
       << setFont( normalFont )     // restore the normal font
       << endHeader();

   prt << startFooter()
       << setFont( footerFont )     // set a font for the footer
       << setPrintPos( 10.0, 3.747, true )
       << "Page: " << pageNumber()  << " of " << numPages()
       << bitMap( bmp, 7.0, 0.0 )
       << setFont( normalFont )     // restore the normal font
       << endFooter();

   prt << "const char *h_font = \"Helvetica\";"                                                 << endl
       << "const char *f_font = \"Times New Roman\";"                                           << endl
       << "const char *n_font = \"Courier New\";"                                               << endl  << endl
       << "const  header = VARIABLE_PITCH | FF_ROMAN;"                                          << endl
       << "const  footer = VARIABLE_PITCH | FF_SWIS;"                                           << endl
       << "const  normal = FIXED_PITCH    | FF_MODERN;"                                         << endl  << endl
       << "// Note: Positive font heights are interpeted as point"                              << endl
       << "//       size (72pts/inch) within the prtStream class"                               << endl  << endl
       << "TFont  headerFont( h_font, 14, 0, 0, 0, FW_BOLD,    header );"                       << endl
       << "TFont  footerFont( f_font, 10, 0, 0, 0, FW_BOLD,    footer );"                       << endl
       << "TFont  normalFont( n_font,  8, 0, 0, 0, FW_REGULAR, normal );"                       << endl  << endl
       << "TResId bmp( \"prtStream\" )      // create a bitmap resource ID"                     << endl  << endl  << endl
       << "spoolfilebuf fbuf;             // create an automagic spool file"                    << endl
       << "prtStream    prt( &fbuf );     // create a  print stream which can handle > 64K"     << endl  << endl
       << "prt << setLeftMargin(   1.00 )"                                                      << endl
       << "    << setTopMargin(    1.00 )"                                                      << endl
		 << "    << setBottomMargin( 1.00 );"                                                     << endl  << endl;

   prt << "prt << startHeader()"                                                                << endl
       << "    << setFont( headerFont )     // set a font for the header"                       << endl
       << "    << setPrintPos(  0.5, 3.188, true )"                                             << endl
       << "    << \"prtStream demo\""                                                           << endl
       << "    << bitMap( bmp, 7.0, 0.0 )   "
       << "// draw bitmap at printers resolution"                                               << endl
       << "    << setFont( normalFont )     // restore the normal font"                         << endl
       << "    << endHeader()"                                                                  << endl  << endl;

   prt << "prt << startFooter()"                                                                << endl
       << "    << setFont( footerFont )     // set a font for the footer"                       << endl
       << "    << setPrintPos( 10.0, 3.747, true )"                                             << endl
       << "    << \"Page: \" << pageNumber()  << \" of \" << numPages()"                        << endl
       << "    << bitMap( bmp, 7.0, 0.0 )   "
       << "// draw bitmap at printers resolution"                                               << endl
       << "    << setFont( normalFont )     // restore the normal font"                         << endl
       << "    << endFooter();"                                                                 << endl  << endl;


   prt << "//tab demo"                                                                          << endl
       << "prt << endl"                                                                         << endl
       << "    << setTabInterval( 0.50 )"                                                       << endl
       << "    << \"Tabbed table:\\t\"     <<  123 << '\\t' <<  456 << '\\t' <<  789 << endl"   << endl
       << "    << \"\\t\\t\"                << 9876 << '\\t' << 6543 << '\\t' << 3210 << endl   << endl" << endl  << endl;

	prt << "// column demo"                                                                      << endl
       << "prt << Column table:\t"                                                              << endl
       << "    << setPrintCol( 2.0 )    << 123  <<  setPrintCol( 2.5 )      << 456 "            << endl
       << "    << setPrintCol( 3.0 )    << 789  <<  endl;"                                      << endl
       << "prt << setPrintCol( 2.0 )    << 9876 <<  setPrintCol( 2.5 )      << 6543 "           << endl
		 << "    << setPrintCol( 3.0 )    << 3210 <<  endl;"                                      << endl  << endl;

   prt << "// draw bitmap at fixed size of 4\" x 1\""                                           << endl
		 << "prt << bitMap( bmp, 2.0, 8.75, 4,0, 1.0 )    << endpg();"                            << endl  << endl;

	// tab demo
	prt << endl
		 << setTabInterval( 0.50 )
		 << "Tabbed table:\t"      <<  123 << '\t' <<  456 << '\t' <<  789 << endl
		 << "\t\t"                 << 9876 << '\t' << 6543 << '\t' << 3210 << endl << endl;

	// column demo
	prt << "Column table:\t"      
		 <<  setPrintCol( 2.0 )    << 123  <<  setPrintCol( 2.5 )      << 456 
		 <<  setPrintCol( 3.0 )    << 789  <<  endl;
	prt <<  setPrintCol( 2.0 )    << 9876 <<  setPrintCol( 2.5 )      << 6543 
		 <<  setPrintCol( 3.0 )    << 3210 <<  endl;

	// draw bitmap at fixed size of 4" x 1"
	prt << bitMap( bmp, 2.0, 8.75, 4.0, 1.0 )    << endpg();

	prt << "drawFrame( 1.00, 0.75, 3.75, 0.25 )"
		 << drawFrame(  1.00, 0.75, 3.75, 0.25 )
		 << setPrintPos(1.75 )
		 << "drawFrame( 1.50, 0.75, 4.25, 0.50, 0.1 )"        << endl
		 << drawFrame(  1.50, 0.75, 4.25, 0.50, 0.1 )
		 << setPrintPos(2.50 )
		 << "drawFrame( 2.25, 0.75, 4.75, 0.50, 0, PS_DOT )"  << endl
       << drawFrame(  2.25, 0.75, 4.75, 0.50, 0, PS_DOT );


   prt << setPrintPos( 3.00 )
		 << "prt << \"drawFrame( 1.00, 0.75, 3.25, 0.25 )\""                                      << endl
       << "    << drawFrame( 1.00, 0.75, 3.25, 0.25 )"                                          << endl
       << "    << ...;"                                                                         << endl  << endl;

   prt << "int   i;"                                                                            << endl
       << "float pos, width;"                                                                   << endl
       << "for ( width=0.000,      pos=6.50,      i=2;  i < 8;"                                 << endl
       << "      width=i * 0.008,  pos=pos+0.50,  i++"                                          << endl
       << "    )  prt << setPrintPos( pos )"                                                    << endl
       << "           << \"line width = \" << width << endl"                                    << endl
       << "           << \"print pos  = \" << pos"                                              << endl
       << "           << drawFrame( pos, 3.00, 2.00, 0.25, width )"                             << endl
       << "           << drawLine(  pos, 3.00, 2.00, 0.25, width );"                            << endl << endl
       << "prt << endpg();"                                                                     << endl << endl
       << "Printer printer;                       // create printer object"                     << endl << endl
       << "if ( printer.ExecPrintDialog( this ) ) // show print dialog"                         << endl
       << "   prt.Print( this, printer );         // print the stream"                          << endl;

   int   i;
   float pos, width;
   for ( width=0.000,      pos=6.00,      i=2;  i < 10;
         width=i * 0.008,  pos=pos+0.50,  i++
       ) prt << setPrintPos( pos )
             << "line width = " << width << endl
             << "print pos  = " << pos
             << drawFrame( pos, 3.00, 2.00, 0.25, width )
             << drawLine(  pos, 3.00, 2.00, 0.25, width );

   prt << endpg();

   Printer printer;                       // create printer object

	if ( printer.ExecPrintDialog( this ) ) // show print dialog
      prt.Print( this, printer );         // print the stream
}


void SDIFrame::Cm1cmexit()
{
   CloseWindow();
}

