
// ************************ BEGIN_COPYRIGHT_NOTICE ************************
//
// This source code copyright (c) 1995 by Eric Dill and Paul Keister
//
// Permission to use, copy, modify, distribute, and sell this software
// and its documentation for any purpose is hereby granted without
// fee, provided that this copyright notice appear in all copies
// and that the author's name not be used in advertising or publicity
// pertaining to distribution of the software without specific, written
// prior permission.  The author makes no representations about the
// suitability of this software for any purpose.  It is provided "as
// is" without express or implied warranty.
//
// Please send all bug fixes, ideas, and comments to:
//
//             Via Email to CompuServe 75061,3310 and 71242,1145
//             Or post to the Borland OWL2:Print/Doc/View Forum on CompuServe
//
// ************************* END_COPYRIGHT_NOTICE *************************

#include "prtstrea.h"


//
// prtStream constructor
//

prtStream::prtStream( streambuf* pStreambuf, float lMargin, float tMargin, float bMargin, float tInterval )
   : iostream( pStreambuf ),
     initLeftMargin(lMargin),    initTopMargin(  tMargin  ),
     initBotMargin( bMargin),    initTabInterval(tInterval)
{
   //
   // initialize prtStream data members
   //

   shouldDelete   = false;

   Parent         = 0;
   Font           = 0;
   xSCALE         = ySCALE    = 1;
   curStreamPos   = 0;
   headerStart    = headerEnd = footerStart = footerEnd   = 0;
   newPage        = newLine   = true;
   leftMargin     = topMargin = printPos    = tabInterval = 0;
   lineLen        = pageNum   = pages       = 0;
   ignoreMargins  = false;
   pdc            = 0;


   //
   // initialize function array to null value
   //

   for( int counter = 0; counter < LastEffect; counter++ )
      RenderFunctions[ counter ] = 0;
}

prtStream::prtStream( float lMargin, float tMargin, float bMargin, float tInterval, streambuf* pStreambuf )
   : iostream(0), ios(0),
     initLeftMargin(lMargin),    initTopMargin(  tMargin  ),
     initBotMargin( bMargin),    initTabInterval(tInterval)
{
   //
   // initialize the base iostream
   //

   if ( !pStreambuf )
      {
         pStreambuf   = new strstreambuf();
         shouldDelete = true;    // remember to delete the streambuf
      }
   else
      shouldDelete = false;      // streambuf supplied by the user -- don't delete it!

   init( pStreambuf );           // initialize the streambuf (now referenced by ios::bp)


   //
   // initialize prtStream data members
   //

   Parent         = 0;
   Font           = 0;
   xSCALE         = ySCALE    = 1;
   curStreamPos   = 0;
   headerStart    = headerEnd = footerStart = footerEnd   = 0;
   newPage        = newLine   = true;
   leftMargin     = topMargin = printPos    = tabInterval = 0;
   lineLen        = pageNum   = pages       = 0;
   ignoreMargins  = false;
   pdc            = 0;


   //
   // initialize function array to null value
   //

   for( int counter = 0; counter < LastEffect; counter++ )
      RenderFunctions[ counter ] = 0;
}


//
// prtStream destructor
//

prtStream::~prtStream( void )
{
   delete Font;

   if ( shouldDelete )
      {
         delete bp;
         bp = 0;
      }
}


//
// prtStream::ConfigureMapMode makes all MapMode adjustments
//

void prtStream::ConfigureMapMode( void )
{
   TPoint   tempSize( pdc->GetDeviceCaps(HORZRES),
                      pdc->GetDeviceCaps(VERTRES) );

   pdc->SetMapMode(   MM_HIENGLISH );     // one logical unit = 0.001 inch
   pdc->SetTextAlign( TA_UPDATECP  );     // update position after text output
   pdc->GetTextMetrics( tm );             // get text metrics
   xSCALE = 1000;
   ySCALE = 1000;


   switch ( pdc->GetMapMode() )
      {
         case MM_TEXT:
            Parent->MessageBox( "MapMode = MM_TEXT", "Warning from prtStream::ConfigureMapMode()" );
            xSCALE = pdc->GetDeviceCaps( LOGPIXELSX );
            ySCALE = pdc->GetDeviceCaps( LOGPIXELSY );
            break;

         case MM_ISOTROPIC:
            Parent->MessageBox( "MapMode = MM_ISOTROPIC", "*** Error in prtStream::ConfigureMapMode() ***" );
            break;

         case MM_ANISOTROPIC:
            Parent->MessageBox( "MapMode = MM_ANISOTROPIC", "*** Error in prtStream::ConfigureMapMode() ***" );
            break;

         case MM_HIENGLISH:
         case MM_LOENGLISH:
         case MM_HIMETRIC:
         case MM_LOMETRIC:
         case MM_TWIPS:
            ySCALE      *= -1;
            tm.tmHeight *= -1;
            break;

         default:
            Parent->MessageBox( "Unknown MapMode", "*** Error in prtStream::ConfigureMapMode() ***" );
            break;
      }

   tabInterval = initTabInterval * xSCALE;

   pdc->DPtoLP( &tempSize );

   printSize = TSize( tempSize );
}


//
// prtStream::PrintLine prints the current line
//

void
prtStream::PrintLine( void )
{
   if ( newLine )
      pdc->MoveTo( leftMargin, printPos );

   if ( lineLen )
      {
         char       *buf            = new char[ lineLen + 1 ];

         streampos   savedStreamPos = tellg();  // save original stream position

         seekg( curStreamPos );

         getline( buf, lineLen+1, '\x00' );

         pdc->TextOut( 0, 0, buf, lineLen );

         curStreamPos = savedStreamPos;         // restore original stream position
         seekg(         savedStreamPos );

         lineLen = 0;

         delete [] buf;
      }
   else
      curStreamPos = tellg();
}

void
prtStream::PrintLine( const char * buffer )   // print the buffer
{
   lineLen = strlen( buffer );
   if ( lineLen )
      {
         pdc->TextOut( 0, 0, buffer, lineLen );
         lineLen = 0;
      }
}


//
// prtStream::NewLine initiates a new line
//

void
prtStream::NewLine( void )
{
   if ( ( !ignoreMargins && pgLimit             - printPos >= 0 ) ||
        (  ignoreMargins && pgLimit + botMargin - printPos >= 0 ) )
      newPage   = true;

   printPos    += tm.tmHeight;
   newLine      = true;
}


//
// prtStream::NewPage initiates a new page
//

void
prtStream::NewPage( void )
{
   PrintLine();
   newPage  =  true;
}


//
// prtStream::SetPrintPos resets the current print position
//
      
void
prtStream::SetPrintPos( float y, float x, bool ignoreMargin )
{
   int      xPos;
   TPoint   pos;

   pdc->GetCurrentPosition( pos );

   if ( x == 0 )
      x = (float) pos.x / xScale();

   if ( y == 0 )
      y = (float) pos.y / yScale();


   if ( x == -2 )                                  // use next tabstop
      xPos = pos.x - (pos.x % tabInterval) + tabInterval;
   else if ( x == -1 ||                            // use left margin
           ( x  < LeftMargin() && !ignoreMargin ) )
      xPos = leftMargin;
   else                                            // use specified x pos
      xPos = x * xScale();

   if ( y < TopMargin() && !ignoreMargin )         // use top margin
      printPos = topMargin;
   else                                            // use specified y pos
      printPos = y * yScale();

   pdc->MoveTo( xPos, printPos );
   newLine = false;
}


//
// prtStream::SetTabInterval resets the tabstop interval
//

void
prtStream::SetTabInterval( float t )
{
   tabInterval = t * xScale();
}


//
// prtStream::SaveState saves a prtStream state value
//

void
prtStream::SaveState( prtStreamState state, int index )
{
   if ( index < 1 || index > LastEffect )
      index = 0;

   int   value;

   switch( state )
      {
         case LeftMargin_:    value  = leftMargin;
                              break;

         case TopMargin_:     value  = topMargin;
                              break;

         case BottomMargin_:  value  = botMargin;
                              break;

         case TabInterval_:   value  = tabInterval;
                              break;

         case PrintPos_:      value  = printPos;
                              break;

         default:             value  = 0;
                              break;
      }

   savedPos[ index ] = value;
}


//
// prtStream::RestoreState restores a prtStream state value
//

void
prtStream::RestoreState( prtStreamState state, int index )
{
   if ( index < 1 || index > LastEffect )
      index = 0;

   int   value = savedPos[ index ];

   switch( state )
      {
         case LeftMargin_:    leftMargin  = value;
                              break;

         case TopMargin_:     topMargin   = value;
                              break;

         case BottomMargin_:  botMargin   = value;
                              break;

         case TabInterval_:   tabInterval = value;
                              break;

         case PrintPos_:      printPos    = value;
                              newLine = true;
                              break;

         default:             // do nothing
                              break;
      }
}


//
// prtStream::SetFont creates a new font
//

void
prtStream::SetFont( LOGFONT& lf )
{
   if ( Font )
      {
         pdc->RestoreFont();
         delete Font;
      }

   Font = new TFont( &lf );
   pdc->SelectObject( *Font );

   int direction = tm.tmHeight < 0 ? -1 : 1;

   pdc->GetTextMetrics( tm );
   tm.tmHeight *= direction;
}


//
// prtStream::SpecialEffect processes a special effect
//

void
prtStream::SpecialEffect( void )
{
   PrintLine();                     // print any text collected to this point

   istream  in( rdbuf() );          // create an input stream formatter
   int      eff;

   in.ignore(1);                    // skip past the special effect header

   in >> eff >> ws;                 // read the special effect function index


   if ( eff >= 0 && eff < LastEffect && in.rdstate() == ios::goodbit )
      if ( RenderFunctions[ eff ] )
         (*RenderFunctions[ eff ])( this, in );

   in.ignore(1);                    // skip cursor past special effect
                              
   curStreamPos = tellg();          // set start of new line
}


//
// prtStreamPage class interfaces with Borland TPrintout class
//

class prtStreamPage : public TPrintout
{
   private:
      prtStream& Prt;

   public:
      prtStreamPage( prtStream& prt, const char* jobName )
         :Prt( prt ), TPrintout( jobName ) {}

      virtual void PrintPage( int page, TRect& rect, unsigned flags );
      virtual bool HasPage( int pageNumber )
         {
            return Prt.good();
         }

      virtual void GetDialogInfo( int& minPage,     int& maxPage,
                                  int& selFromPage, int& selToPage )
         {
            TPrintout::GetDialogInfo( minPage,     maxPage,
                                      selFromPage, selToPage );
            maxPage     = Prt.Pages();
         }
};


//
// prtStream::Print starts the print process
//

void
prtStream::Print( TWindow *currentWin, TPrinter& printer, int nPages, const char* jobName )
{
   Parent = currentWin;

   seekg(0);   // set input from start of buffer

   long  savedFlags = flags();   // save format state
   flags( ios::skipws );         // default format state

   if( nPages != 0 )    pages  = nPages;
                              
   printer.Print( currentWin, prtStreamPage( *this, jobName ), false );

   flags( savedFlags );          // restore format state
}


//
// prtStreamPage::PrintPage is the TPrintout interface
//

void
prtStreamPage::PrintPage( int page, TRect&, unsigned )
{
   Prt.pdc = DC;
   Prt.PrintPage( page );
}


//
// prtStream::PrintPage prints a page
//

void
prtStream::PrintPage( int page )
{
   ConfigureMapMode();

   if ( page == 1 )
      {
         leftMargin  = initLeftMargin  * xSCALE;
         topMargin   = initTopMargin   * ySCALE;
         botMargin   = initBotMargin   * ySCALE;
      }

   pdc->MoveTo( leftMargin, topMargin );  // set start position

   printPos = topMargin;                  // printPos decreases from top of page
   lineLen  = 0;                          // at beginning of line
   pageNum  = page;                       // current page number

   pgLimit  = printSize.cy - botMargin;   // printable (vertical) region

   newPage      = false;                  // don't force a page break
   curStreamPos = tellg();                // mark current stream position

   Scan( headerStart, headerEnd );        // process page header

   while( good() && newPage == false )    // ? more page data ?
      {
         ReadStream();                    //    process stream data
      }

   Scan( footerStart, footerEnd );        // process page footer
}


void prtStream::Scan( streampos start, streampos pastEnd )
{
   int         savedPrintPos  = printPos;    // save current print pos
   streampos   savedStreamPos = tellg();     // save current stream position

   ignoreMargins = true;

   if( start )                               // ? start pos given ?
      {
         seekg( start );                     //    position stream at start

         if( !pastEnd )                      //    ? end pos missing ?
            pastEnd = tellp();               //       set end pos to the end of the stream

         while( tellg() < pastEnd )          //    ? more stream data to process ?
            ReadStream();                    //       process stream data
      }

   curStreamPos =  savedStreamPos;           // restore saved stream position
   seekg(    savedStreamPos );

   printPos = savedPrintPos;                 // restore saved print pos
   newLine  = true;

   ignoreMargins = false;
}


void prtStream::ReadStream( void )
{
   char ch;             // char buffer
   get( ch );           // read initial char


   //
   // two escape characters signal a special effect
   //

   if( ch != ESC || peek() != ESC )
      {
         if ( ch == '\n' )             // ? new line ?
            {
               PrintLine();            //    print accumulated text
               NewLine();              //    signal a new line
            }
         else if ( ch == '\t' )        // ? tab ?
            {
               PrintLine();            //    print accumulated text
               SetPrintPos( 0, -2 );   //    skip to next tab stop
            }
         else                          // ? normal text ?
             ++lineLen;                //    accumulate text
      }
   else                                // ? special effect ?
      {
         SpecialEffect();              //    process the special effect
      }
}

