
// ************************ BEGIN_COPYRIGHT_NOTICE ************************
//
// This source code copyright (c) 1995 by Eric Dill and Paul Keister
//
// Permission to use, copy, modify, distribute, and sell this software
// and its documentation for any purpose is hereby granted without
// fee, provided that this copyright notice appear in all copies
// and that the author's name not be used in advertising or publicity
// pertaining to distribution of the software without specific, written
// prior permission.  The author makes no representations about the
// suitability of this software for any purpose.  It is provided "as
// is" without express or implied warranty.
//
// Please send all bug fixes, ideas, and comments to:
//
//             Via Email to CompuServe 75061,3310 and 71242,1145
//             Or post to the Borland OWL2:Print/Doc/View Forum on CompuServe
//
// ************************* END_COPYRIGHT_NOTICE *************************

// =>RESOURCE REQUIREMENTS
//
// Because this version of the ptrStream library uses the Borland
// OWL printing classes, the resource file \include\owl\printer.rc
// must be included in your build
//

#ifndef __PRTSTREAM__H
#define __PRTSTREAM__H


//
// provided for compatibily with BC++ 4.0x
//

#if __BORLANDC__ < 0x460
   #define  bool  BOOL
   #define  false FALSE
   #define  true  TRUE
#endif


#include <strstrea.h>
#include <iomanip.h>
#include <string.h>
#include <stdio.h>

#include <owl\module.h>
#include <owl\dc.h>
#include <owl\printer.h>


#define  ESC            ( (char) 0x1b )
#define  LMARGIN_W      6


#define WHITESP         " "
#define defTopM          0.5
#define defBotM          0.5
#define defLeftM         0.5
#define defTabInterval   0.5
#define defPgLen        11.0
#define defPrtLimit     defPgLen-defTopM

//
// Special effects objects
//

enum SpecialEffect
   {
      Box,           Frame,         Line,
      BitMap,        Font,
      LeftMargin,    TopMargin,
      TabInterval,   BottomMargin,
      SetPrintPos,   SetPrintCol,
      SaveState,     RestoreState,
      PageBreak,     CondPageBreak,
      StartHeader,   EndHeader,     PrtPageNum,
      StartFooter,   EndFooter,     PrtNumPages,
      LastEffect
   };


enum prtStreamState
   {
      LeftMargin_,   TopMargin_, BottomMargin_, PrintPos_,  TabInterval_
   };


//
// prtStream class defininition
//

class prtStream : public iostream
{
   //
   // margins expressed in inches externally, 0.001 inches internally
   // all positions are relative to upper left corner of printable area
   //

   public:

      //
      // constructor / destructor
      //

      prtStream( float leftMargin = defLeftM, float topMargin   = defTopM,
                 float botMargin  = defBotM,  float tabInterval = defTabInterval,
                 streambuf* pStreambuf = 0 );
      prtStream( streambuf* pStreambuf, float leftMargin = defLeftM, float topMargin   = defTopM,
                 float botMargin  = defBotM,  float tabInterval = defTabInterval );

      virtual ~prtStream( void );


      //
      // public member functions for general use
      //

      void     Print( TWindow* currentWin, TPrinter& printer, int nPages = 0,
                      const char* jobName = "prtStream print job" );


      //
      // These functions are for use by the SpecialEffect<..>::Render()
      // functions and would be private with friend templates
      //

      void  PrintLine(     void );
      void  PrintLine(     const char *buff );
      void  NewLine(       void );
      void  NewPage(       void );
      void  SaveState(     prtStreamState state = PrintPos_, int index = 0 );
      void  RestoreState(  prtStreamState state = PrintPos_, int index = 0 );
      void  SetPrintPos(   float y, float x = -1, bool ignoreMargin = false );
      void  SetTabInterval(float t );
      void  SetFont(       LOGFONT& lf );

      TPrintDC&   PDC(        void ) { return *pdc;     }

      const int   yPrintPos(  void ) { return printPos; }
      const int   xPrintPos(  void ) { TPoint pos;
                                       pdc->GetCurrentPosition( pos );
                                       return pos.x;
                                     }

      const int xScale(       void ) { return xSCALE; }
      const int yScale(       void ) { return ySCALE; }

      void        NextPage(   void ) {      ++pages;   }
      const int   Pages(      void ) { return pages;   }
      const int   PageNum(    void ) { return pageNum; }
      const float TopMargin(  void ) { return float(topMargin)  / ySCALE; }
      const float BotMargin(  void ) { return float(botMargin)  / ySCALE; }
      const float LeftMargin( void ) { return float(leftMargin) / xSCALE; }
      const int   LeftMargin( int  ) { return leftMargin; }

      void        IgnoreMargins( bool ignore = false )   { ignoreMargins = ignore;  }

      void        LeftMargin( float margin )    { initLeftMargin = leftMargin = margin * xSCALE;  }

      void        TopMargin(  float margin )    { initTopMargin  = topMargin  = margin * ySCALE;
                                                   if ( float(printPos) / ySCALE < margin )
                                                      SetPrintPos( margin, 0 );
                                                }

      void        BotMargin(  float margin )    { initBotMargin = botMargin   = margin * ySCALE;  }

      void        HeaderStart( streampos ptr )  { headerStart = ptr; }
      void        HeaderEnd(   streampos ptr )  { headerEnd   = ptr; }
      void        FooterStart( streampos ptr )  { footerStart = ptr; }
      void        FooterEnd(   streampos ptr )  { footerEnd   = ptr; }


   private:
      prtStream( prtStream& );            // no copying    allowed!
      operator=( prtStream& );            // no assignment allowed!

      bool        shouldDelete;           // delete the streambuf?
      TEXTMETRIC  tm;
      TFont      *Font;
      TWindow    *Parent;
      TPrintDC   *pdc;
      TSize       printSize;
      float       initLeftMargin,
                  initTopMargin,
                  initBotMargin,
                  initTabInterval;
      int         xSCALE,                 // units per inch along the x-axis
                  ySCALE,                 // units per inch along the y-axis
                  printPos,
                  leftMargin,
                  topMargin,
                  botMargin,
                  tabInterval,
                  lineLen,
                  pages,
                  pageNum,
                  pgLimit,
                  savedPos[ LastEffect ];
      bool        newLine,
                  newPage,
                  ignoreMargins;
      streampos   curStreamPos,
                  headerStart,   headerEnd,
                  footerStart,   footerEnd;

      void   ConfigureMapMode( void );
      void   SpecialEffect(    void );
      void   PrintPage( int page );

      void   ReadStream( void );
      void   Scan( streampos start, streampos pastEnd );

      friend class prtStreamPage;


    public:

      //
      // dispatch table for special effect functions
      //    would be private with friend templates
      //

      void (*RenderFunctions[ LastEffect ])( prtStream *pclass, istream& in );
};


#include "prtManip.h"
#include "spoolbuf.h"

#endif //  __PRTSTREAM__H

